from pydantic import BaseModel, Field
from typing import Any


class BaseResponse(BaseModel):
    status: str
    content: Any
    credits_used: int | None = None
    credits_remaining: int | None = None
    tips: str | None = None


class ConversionParams(BaseModel):
    url: str = Field(
        description="URL to the source file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files.",
        default="",
    )
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    )
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    )
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    )
    unwrap: bool = Field(
        description="Unwrap lines into a single line within table cells when lineGrouping is enabled. Must be true or false. (Optional)",
        default=False,
    )
    rect: str = Field(
        description="Defines coordinates for extraction (e.g., '51.8,114.8,235.5,204.0'). (Optional)",
        default="",
    )
    lang: str = Field(
        description="Language for OCR for scanned documents. Default is 'eng'. See PDF.co docs for supported languages. (Optional, Default: 'eng')",
        default="eng",
    )
    line_grouping: str = Field(
        description="Enables line grouping within table cells when set to '1'. (Optional)",
        default="0",
    )
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    )
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    )
    autosize: bool = Field(
        description="Controls automatic page sizing. If true, page dimensions adjust to content. If false, uses worksheet’s page setup. (Optional)",
        default=False,
    )

    html: str = Field(
        description="Input HTML code to be converted. To convert the link to a PDF use the /pdf/convert/from/url endpoint instead.",
        default="",
    )
    templateId: str = Field(
        description="Set to the ID of your HTML template. You can find and copy the ID from HTML to PDF Templates.",
        default="",
    )
    templateData: str = Field(
        description="Set it to a string with input JSON data (recommended) or CSV data.",
        default="",
    )
    margins: str = Field(
        description="Set to CSS style margins like 10px, 5mm, 5in for all sides or 5px 5px 5px 5px (the order of margins is top, right, bottom, left). (Optional)",
        default="",
    )
    paperSize: str = Field(
        description="A4 is set by default. Can be Letter, Legal, Tabloid, Ledger, A0, A1, A2, A3, A4, A5, A6 or a custom size. Custom size can be set in px (pixels), mm or in (inches) with width and height separated by space like this: 200 300, 200px 300px, 200mm 300mm, 20cm 30cm or 6in 8in. (Optional)",
        default="",
    )
    orientation: str = Field(
        description="Set to Portrait or Landscape. Portrait is set by default. (Optional)",
        default="",
    )
    printBackground: bool = Field(
        description="true by default. Set to false to disable printing of background. (Optional)",
        default=True,
    )
    mediaType: str = Field(
        description="Uses print by default. Set to screen to convert HTML as it appears in a browser or print to convert as it appears for printing or none to set none as mediaType for CSS styles. (Optional)",
        default="",
    )
    DoNotWaitFullLoad: bool = Field(
        description="false by default. Set to true to skip waiting for full load (like full video load etc. that may affect the total conversion time). (Optional)",
        default=False,
    )
    header: str = Field(
        description="User definable HTML for the header to be applied on every page header. (Optional)",
        default="",
    )
    footer: str = Field(
        description="User definable HTML for the footer to be applied on every page footer. (Optional)",
        default="",
    )

    worksheetIndex: str = Field(
        description="Index of the worksheet to convert. (Optional)", default=""
    )

    def parse_payload(self, async_mode: bool = True):
        payload = {
            "async": async_mode,
        }
        if self.url:
            payload["url"] = self.url
        if self.httpusername:
            payload["httpusername"] = self.httpusername
        if self.httppassword:
            payload["httppassword"] = self.httppassword
        if self.pages:
            payload["pages"] = self.pages
        if self.unwrap:
            payload["unwrap"] = self.unwrap
        if self.rect:
            payload["rect"] = self.rect
        if self.lang:
            payload["lang"] = self.lang
        if self.line_grouping:
            payload["lineGrouping"] = self.line_grouping
        if self.password:
            payload["password"] = self.password
        if self.name:
            payload["name"] = self.name
        if self.autosize:
            payload["autosize"] = self.autosize

        if self.html:
            payload["html"] = self.html
        if self.templateId:
            payload["templateId"] = self.templateId
        if self.templateData:
            payload["templateData"] = self.templateData
        if self.margins:
            payload["margins"] = self.margins
        if self.paperSize:
            payload["paperSize"] = self.paperSize
        if self.orientation:
            payload["orientation"] = self.orientation
        if self.printBackground:
            payload["printBackground"] = self.printBackground
        if self.mediaType:
            payload["mediaType"] = self.mediaType
        if self.DoNotWaitFullLoad:
            payload["DoNotWaitFullLoad"] = self.DoNotWaitFullLoad
        if self.header:
            payload["header"] = self.header
        if self.footer:
            payload["footer"] = self.footer

        if self.worksheetIndex:
            payload["worksheetIndex"] = self.worksheetIndex

        return payload



from contextlib import asynccontextmanager
from httpx import AsyncClient
import os
import sys
from typing import AsyncGenerator
import importlib.metadata

__BASE_URL = "https://api.pdf.co"
X_API_KEY = os.getenv("X_API_KEY")

__version__ = importlib.metadata.version("pdfco-mcp")
print(f"pdfco-mcp version: {__version__}", file=sys.stderr)


@asynccontextmanager
async def PDFCoClient(api_key: str | None = None) -> AsyncGenerator[AsyncClient, None]:
    # Use provided API key, fall back to environment variable
    x_api_key = api_key or X_API_KEY

    if not x_api_key:
        raise ValueError("""API key is required. Please provide an API key as a parameter or set X_API_KEY in the environment variables.
        
        To get the API key:
        1. Sign up at https://pdf.co
        2. Get the API key from the dashboard
        3. Either set it as an environment variable or provide it as a parameter
        
        Environment variable setup example (.cursor/mcp.json):
        ```json
        {
            "mcpServers": {
                "pdfco": {
                    "command": "uvx",
                    "args": [
                        "pdfco-mcp"
                    ],
                    "env": {
                        "X_API_KEY": "YOUR_API_KEY"
                    }
                }
            }
        }
        ```
        
        Or provide the API key as a parameter when calling the tool.
        """)

    client = AsyncClient(
        base_url=__BASE_URL,
        headers={
            "x-api-key": x_api_key,
            "User-Agent": f"pdfco-mcp/{__version__}",
        },
    )
    try:
        yield client
    finally:
        await client.aclose()



from pdfco.mcp.server import mcp
from pdfco.mcp.services.pdf import pdf_add
from pdfco.mcp.models import BaseResponse, ConversionParams

from pydantic import Field
from typing import List, Any


@mcp.tool()
async def pdf_add_annotations_images_fields(
    url: str = Field(
        description="URL to the source PDF file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    password: str = Field(
        description="Password for the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    annotations: List[Any] = Field(
        description="Array of annotation objects to add text, links, shapes, etc. Each object can have: 'text' (string), 'x' (number), 'y' (number), 'size' (number), 'pages' (string), 'color' (string hex), 'link' (string URL), 'fontName' (string), 'fontItalic' (boolean), 'fontBold' (boolean), 'fontStrikeout' (boolean), 'fontUnderline' (boolean). (Optional)",
        default=[],
    ),
    images: List[Any] = Field(
        description="Array of image objects to add images to PDF. Each object can have: 'url' (string), 'x' (number), 'y' (number), 'width' (number), 'height' (number), 'pages' (string). (Optional)",
        default=[],
    ),
    fields: List[Any] = Field(
        description="Array of form field objects to fill PDF form fields. Each object can have: 'fieldName' (string), 'pages' (string), 'text' (string), 'fontName' (string), 'size' (number), 'fontBold' (boolean), 'fontItalic' (boolean), 'fontStrikeout' (boolean), 'fontUnderline' (boolean). (Optional)",
        default=[],
    ),
    expiration: int = Field(
        description="Set the expiration time for the output link in minutes. After this specified duration, any generated output file(s) will be automatically deleted. (Optional)",
        default=60,
    ),
    encrypt: bool = Field(description="Encrypt output file. (Optional)", default=False),
    flatten: bool = Field(
        description="Flatten filled form fields and annotations into PDF content. Set to true to disable editing of filled form fields in the output PDF. (Optional)",
        default=False,
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Add text, images, forms, other PDFs, fill forms, links to external sites and external PDF files. You can update or modify PDF and scanned PDF files.

    This tool supports three main ways to add content:

    1. **annotations**: Add text, links, shapes, etc.
       Properties: text, x, y, size, pages, color, link, fontName, fontItalic, fontBold, fontStrikeout, fontUnderline

    2. **images**: Add images or other PDF content
       Properties: url, x, y, width, height, pages

    3. **fields**: Fill existing form fields
       Properties: fieldName, pages, text, fontName, size, fontBold, fontItalic, fontStrikeout, fontUnderline

    Example annotations:
    [{"text": "Sample Text - Click here to test link", "x": 250, "y": 240, "size": 24, "pages": "0-", "color": "CCBBAA", "link": "https://pdf.co/", "fontName": "Comic Sans MS", "fontItalic": true, "fontBold": true, "fontStrikeout": false, "fontUnderline": true}]

    Example images:
    [{"url": "https://pdfco-test-files.s3.us-west-2.amazonaws.com/pdf-edit/logo.png", "x": 270, "y": 150, "width": 159, "height": 43, "pages": "0"}]

    Example fields:
    [{"fieldName": "topmostSubform[0].Page1[0].YourSocial_ReadOrderControl[0].f1_05[0]", "pages": "1", "text": "Joan B.", "fontName": "Arial", "size": 6, "fontBold": true, "fontItalic": true, "fontStrikeout": true, "fontUnderline": true}]

    Ref: https://developer.pdf.co/api-reference/pdf-add.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername,
        httppassword=httppassword,
        password=password,
        name=name,
        expiration=expiration,
    )

    # Prepare additional parameters
    add_params = {}

    if annotations:
        add_params["annotations"] = annotations
    if images:
        add_params["images"] = images
    if fields:
        add_params["fields"] = fields
    if encrypt:
        add_params["encrypt"] = encrypt
    if flatten:
        add_params["flatten"] = flatten

    return await pdf_add(params, **add_params, api_key=api_key)



from pdfco.mcp.server import mcp
from pdfco.mcp.services.pdf import make_pdf_searchable, make_pdf_unsearchable
from pdfco.mcp.models import BaseResponse, ConversionParams

from pydantic import Field


@mcp.tool()
async def pdf_make_searchable(
    url: str = Field(
        description="URL to the source file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    lang: str = Field(
        description="Language for OCR for scanned documents. Default is 'eng'. See PDF.co docs for supported languages. (Optional, Default: 'eng')",
        default="eng",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert scanned PDF documents or image files into a text-searchable PDF.
    Runs OCR and adds an invisible text layer that can be used for text search.
    Ref: https://developer.pdf.co/api-reference/pdf-change-text-searchable/searchable.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername,
        httppassword=httppassword,
        lang=lang,
        pages=pages,
        password=password,
        name=name,
    )

    return await make_pdf_searchable(params, api_key=api_key)


@mcp.tool()
async def pdf_make_unsearchable(
    url: str = Field(
        description="URL to the source file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Make existing PDF document non-searchable by removing the text layer from it.
    Ref: https://developer.pdf.co/api-reference/pdf-change-text-searchable/unsearchable.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername,
        httppassword=httppassword,
        pages=pages,
        password=password,
        name=name,
    )

    return await make_pdf_unsearchable(params, api_key=api_key)



from pdfco.mcp.server import mcp
from pdfco.mcp.services.client import PDFCoClient
from pdfco.mcp.models import BaseResponse

from pydantic import Field


@mcp.tool()
async def upload_file(
    file_path: str = Field(description="The absolute path to the file to upload"),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Upload a file to the PDF.co API
    """
    try:
        async with PDFCoClient(api_key=api_key) as client:
            response = await client.post(
                "/v1/file/upload",
                files={
                    "file": open(file_path, "rb"),
                },
            )
            res = response.json()
            return BaseResponse(
                status="success" if res["status"] == 200 else "error",
                content=res,
                tips=f"You can use the url {res['url']} to access the file",
            )
    except Exception as e:
        return BaseResponse(
            status="error",
            content=str(e),
        )



from pdfco.mcp.server import mcp
from pdfco.mcp.services.pdf import parse_invoice, extract_pdf_attachments
from pdfco.mcp.models import BaseResponse, ConversionParams

from pydantic import Field


@mcp.tool()
async def ai_invoice_parser(
    url: str = Field(
        description="URL to the source PDF file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    AI Invoice Parser: Extracts data from invoices using AI.
    Ref: https://developer.pdf.co/api-reference/ai-invoice-parser.md
    """

    # Pass arguments directly; ConversionParams now handles str with default=None
    params = ConversionParams(
        url=url,
    )

    return await parse_invoice(params, api_key=api_key)


@mcp.tool()
async def extract_attachments(
    url: str = Field(description="URL to the source PDF file."),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    password: str = Field(description="Password of PDF file. (Optional)", default=""),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Extracts attachments from a source PDF file.
    Ref: https://developer.pdf.co/api-reference/pdf-extract-attachments.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername if httpusername else None,
        httppassword=httppassword if httppassword else None,
        password=password if password else None,
    )
    return await extract_pdf_attachments(params, api_key=api_key)



import asyncio
import time
from pdfco.mcp.server import mcp
from pdfco.mcp.services.client import PDFCoClient
from pdfco.mcp.models import BaseResponse

from pydantic import Field


async def _get_job_status(job_id: str, api_key: str = "") -> BaseResponse:
    """
    Internal helper function to check job status without MCP tool decoration
    """
    try:
        async with PDFCoClient(api_key=api_key) as client:
            response = await client.post(
                "/v1/job/check",
                json={
                    "jobId": job_id,
                },
            )
            json_data = response.json()
            return BaseResponse(
                status=json_data["status"],
                content=json_data,
                credits_used=json_data.get("credits"),
                credits_remaining=json_data.get("remainingCredits"),
                tips="You can download the result if status is success",
            )
    except Exception as e:
        return BaseResponse(
            status="error",
            content=str(e),
        )


@mcp.tool()
async def get_job_check(
    job_id: str = Field(description="The ID of the job to get the status of"),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Check the status and results of a job
    Status can be:
    - working: background job is currently in work or does not exist.
    - success: background job was successfully finished.
    - failed: background job failed for some reason (see message for more details).
    - aborted: background job was aborted.
    - unknown: unknown background job id. Available only when force is set to true for input request.
    """
    return await _get_job_status(job_id, api_key)


@mcp.tool()
async def wait_job_completion(
    job_id: str = Field(description="The ID of the job to get the status of"),
    interval: int = Field(
        description="The interval to check the status of the job (seconds)", default=1
    ),
    timeout: int = Field(
        description="The timeout to wait for the job to complete (seconds)", default=300
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Wait for a job to complete
    """
    start_time = time.time()
    job_check_count = 0
    credits_used = 0
    credits_remaining = 0
    while True:
        response = await _get_job_status(job_id, api_key=api_key)
        job_check_count += 1
        credits_used += response.credits_used or 0
        credits_remaining = response.credits_remaining or 0
        if response.status == "success":
            return BaseResponse(
                status="success",
                content=response.content,
                credits_used=credits_used,
                credits_remaining=credits_remaining,
                tips=f"Job check count: {job_check_count}",
            )
        elif response.status == "failed":
            return BaseResponse(
                status="error",
                content=response.content,
                credits_used=credits_used,
                credits_remaining=credits_remaining,
            )
        await asyncio.sleep(interval)
        if time.time() - start_time > timeout:
            return BaseResponse(
                status="error",
                content="Job timed out",
                credits_used=credits_used,
                credits_remaining=credits_remaining,
                tips=f"Job check count: {job_check_count}",
            )



from pdfco.mcp.server import mcp
from pdfco.mcp.services.pdf import convert_to, convert_from
from pdfco.mcp.models import BaseResponse, ConversionParams

from pydantic import Field


@mcp.tool()
async def pdf_to_json(
    url: str = Field(
        description="URL to the source file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    unwrap: bool = Field(
        description="Unwrap lines into a single line within table cells when lineGrouping is enabled. Must be true or false. (Optional)",
        default=False,
    ),
    rect: str = Field(
        description="Defines coordinates for extraction (e.g., '51.8,114.8,235.5,204.0'). (Optional)",
        default="",
    ),
    lang: str = Field(
        description="Language for OCR for scanned documents. Default is 'eng'. See PDF.co docs for supported languages. (Optional, Default: 'eng')",
        default="eng",
    ),
    line_grouping: str = Field(
        description="Enables line grouping within table cells when set to '1'. (Optional)",
        default="0",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert PDF and scanned images into JSON representation with text, fonts, images, vectors, and formatting preserved using the /pdf/convert/to/json2 endpoint.
    Ref: https://developer.pdf.co/api-reference/pdf-to-json/basic.md
    """
    return await convert_to(
        "pdf",
        "json2",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            pages=pages,
            unwrap=unwrap,
            rect=rect,
            lang=lang,
            line_grouping=line_grouping,
            password=password,
            name=name,
        ),
        api_key=api_key,
    )


@mcp.tool()
async def pdf_to_csv(
    url: str = Field(
        description="URL to the source file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    unwrap: bool = Field(
        description="Unwrap lines into a single line within table cells when lineGrouping is enabled. Must be true or false. (Optional)",
        default=False,
    ),
    rect: str = Field(
        description="Defines coordinates for extraction (e.g., '51.8,114.8,235.5,204.0'). (Optional)",
        default="",
    ),
    lang: str = Field(
        description="Language for OCR for scanned documents. Default is 'eng'. See PDF.co docs for supported languages. (Optional, Default: 'eng')",
        default="eng",
    ),
    line_grouping: str = Field(
        description="Enables line grouping within table cells when set to '1'. (Optional)",
        default="0",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert PDF and scanned images into CSV representation with layout, columns, rows, and tables.
    Ref: https://developer.pdf.co/api-reference/pdf-to-csv.md
    """
    return await convert_to(
        "pdf",
        "csv",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            pages=pages,
            unwrap=unwrap,
            rect=rect,
            lang=lang,
            line_grouping=line_grouping,
            password=password,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def pdf_to_text(
    url: str = Field(
        description="URL to the source file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    unwrap: bool = Field(
        description="Unwrap lines into a single line within table cells when lineGrouping is enabled. Must be true or false. (Optional)",
        default=False,
    ),
    rect: str = Field(
        description="Defines coordinates for extraction (e.g., '51.8,114.8,235.5,204.0'). (Optional)",
        default="",
    ),
    lang: str = Field(
        description="Language for OCR for scanned documents. Default is 'eng'. See PDF.co docs for supported languages. (Optional, Default: 'eng')",
        default="eng",
    ),
    line_grouping: str = Field(
        description="Enables line grouping within table cells when set to '1'. (Optional)",
        default="0",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert PDF and scanned images to text with layout preserved.
    Ref: https://developer.pdf.co/api-reference/pdf-to-text/basic.md
    """
    return await convert_to(
        "pdf",
        "text",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            pages=pages,
            unwrap=unwrap,
            rect=rect,
            lang=lang,
            line_grouping=line_grouping,
            password=password,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def pdf_to_xls(
    url: str = Field(
        description="URL to the source file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    unwrap: bool = Field(
        description="Unwrap lines into a single line within table cells when lineGrouping is enabled. Must be true or false. (Optional)",
        default=False,
    ),
    rect: str = Field(
        description="Defines coordinates for extraction (e.g., '51.8,114.8,235.5,204.0'). (Optional)",
        default="",
    ),
    lang: str = Field(
        description="Language for OCR for scanned documents. Default is 'eng'. See PDF.co docs for supported languages. (Optional, Default: 'eng')",
        default="eng",
    ),
    line_grouping: str = Field(
        description="Enables line grouping within table cells when set to '1'. (Optional)",
        default="0",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert PDF and scanned images to XLS (Excel 97-2003) format.
    Ref: https://developer.pdf.co/api-reference/pdf-to-excel/xls.md
    """
    return await convert_to(
        "pdf",
        "xls",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            pages=pages,
            unwrap=unwrap,
            rect=rect,
            lang=lang,
            line_grouping=line_grouping,
            password=password,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def pdf_to_xlsx(
    url: str = Field(
        description="URL to the source file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    unwrap: bool = Field(
        description="Unwrap lines into a single line within table cells when lineGrouping is enabled. Must be true or false. (Optional)",
        default=False,
    ),
    rect: str = Field(
        description="Defines coordinates for extraction (e.g., '51.8,114.8,235.5,204.0'). (Optional)",
        default="",
    ),
    lang: str = Field(
        description="Language for OCR for scanned documents. Default is 'eng'. See PDF.co docs for supported languages. (Optional, Default: 'eng')",
        default="eng",
    ),
    line_grouping: str = Field(
        description="Enables line grouping within table cells when set to '1'. (Optional)",
        default="0",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert PDF and scanned images to XLSX (Excel 2007+) format.
    Ref: https://developer.pdf.co/api-reference/pdf-to-excel/xlsx.md
    """
    return await convert_to(
        "pdf",
        "xlsx",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            pages=pages,
            unwrap=unwrap,
            rect=rect,
            lang=lang,
            line_grouping=line_grouping,
            password=password,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def pdf_to_xml(
    url: str = Field(
        description="URL to the source file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    unwrap: bool = Field(
        description="Unwrap lines into a single line within table cells when lineGrouping is enabled. Must be true or false. (Optional)",
        default=False,
    ),
    rect: str = Field(
        description="Defines coordinates for extraction (e.g., '51.8,114.8,235.5,204.0'). (Optional)",
        default="",
    ),
    lang: str = Field(
        description="Language for OCR for scanned documents. Default is 'eng'. See PDF.co docs for supported languages. (Optional, Default: 'eng')",
        default="eng",
    ),
    line_grouping: str = Field(
        description="Enables line grouping within table cells when set to '1'. (Optional)",
        default="0",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert PDF and scanned images to XML format.
    Ref: https://developer.pdf.co/api-reference/pdf-to-xml.md
    """
    return await convert_to(
        "pdf",
        "xml",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            pages=pages,
            unwrap=unwrap,
            rect=rect,
            lang=lang,
            line_grouping=line_grouping,
            password=password,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def pdf_to_html(
    url: str = Field(
        description="URL to the source file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    unwrap: bool = Field(
        description="Unwrap lines into a single line within table cells when lineGrouping is enabled. Must be true or false. (Optional)",
        default=False,
    ),
    rect: str = Field(
        description="Defines coordinates for extraction (e.g., '51.8,114.8,235.5,204.0'). (Optional)",
        default="",
    ),
    lang: str = Field(
        description="Language for OCR for scanned documents. Default is 'eng'. See PDF.co docs for supported languages. (Optional, Default: 'eng')",
        default="eng",
    ),
    line_grouping: str = Field(
        description="Enables line grouping within table cells when set to '1'. (Optional)",
        default="0",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert PDF and scanned images to HTML format.
    Ref: https://developer.pdf.co/api-reference/pdf-to-html.md
    """
    return await convert_to(
        "pdf",
        "html",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            pages=pages,
            unwrap=unwrap,
            rect=rect,
            lang=lang,
            line_grouping=line_grouping,
            password=password,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def pdf_to_image(
    url: str = Field(
        description="URL to the source file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    unwrap: bool = Field(
        description="Unwrap lines into a single line within table cells when lineGrouping is enabled. Must be true or false. (Optional)",
        default=False,
    ),
    rect: str = Field(
        description="Defines coordinates for extraction (e.g., '51.8,114.8,235.5,204.0'). (Optional)",
        default="",
    ),
    lang: str = Field(
        description="Language for OCR for scanned documents. Default is 'eng'. See PDF.co docs for supported languages. (Optional, Default: 'eng')",
        default="eng",
    ),
    line_grouping: str = Field(
        description="Enables line grouping within table cells when set to '1'. (Optional)",
        default="0",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    type: str = Field(
        description="Type of image to convert to. (jpg, png, webp, tiff) (Optional)",
        default="jpg",
        choices=["jpg", "png", "webp", "tiff"],
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert PDF and scanned images to various image formats (JPG, PNG, WebP, TIFF).
    Ref:
     - https://developer.pdf.co/api-reference/pdf-to-image/jpg.md
     - https://developer.pdf.co/api-reference/pdf-to-image/png.md
     - https://developer.pdf.co/api-reference/pdf-to-image/webp.md
     - https://developer.pdf.co/api-reference/pdf-to-image/tiff.md
    """
    return await convert_to(
        "pdf",
        type,
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            pages=pages,
            unwrap=unwrap,
            rect=rect,
            lang=lang,
            line_grouping=line_grouping,
            password=password,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def document_to_pdf(
    url: str = Field(
        description="URL to the source file (DOC, DOCX, RTF, TXT, XPS). Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    autosize: bool = Field(
        description="Controls automatic page sizing. If true, page dimensions adjust to content. If false, uses worksheet’s page setup. (Optional)",
        default=False,
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert various document types (DOC, DOCX, RTF, TXT, XLS, XLSX, CSV, HTML, JPG, PNG, TIFF, WEBP) into PDF.
    Ref: https://developer.pdf.co/api-reference/pdf-from-document/doc.md
    """
    return await convert_from(
        "pdf",
        "doc",
        ConversionParams(
            url=url,
            autosize=autosize,
            httpusername=httpusername,
            httppassword=httppassword,
            pages=pages,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def csv_to_pdf(
    url: str = Field(
        description="URL to the source file (CSV, XLS, XLSX). Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    autosize: bool = Field(
        description="Controls automatic page sizing. If true, page dimensions adjust to content. If false, uses worksheet’s page setup. (Optional)",
        default=False,
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert CSV or spreadsheet files (XLS, XLSX) to PDF.
    Ref: https://developer.pdf.co/api-reference/pdf-from-document/csv.md
    """
    return await convert_from(
        "pdf",
        "csv",
        ConversionParams(
            url=url,
            autosize=autosize,
            httpusername=httpusername,
            httppassword=httppassword,
            pages=pages,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def image_to_pdf(
    url: str = Field(
        description="URL to the source file (JPG, PNG, TIFF). Multiple files are supported (by providing a comma-separated list of URLs). Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated page indices (e.g., '0, 1, 2-' or '1, 3-7'). Use '!' for inverted page numbers (e.g., '!0' for last page). Processes all pages if None. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert various image formats (JPG, PNG, TIFF) to PDF.
    Ref: https://developer.pdf.co/api-reference/pdf-from-image.md
    ```
    """
    return await convert_from(
        "pdf",
        "image",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            pages=pages,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def webpage_to_pdf(
    url: str = Field(description="URL to the source file (external webpage URL)."),
    margins: str = Field(
        description="Set to CSS style margins like 10px, 5mm, 5in for all sides or 5px 5px 5px 5px (the order of margins is top, right, bottom, left). (Optional)",
        default="",
    ),
    paperSize: str = Field(
        description="A4 is set by default. Can be Letter, Legal, Tabloid, Ledger, A0, A1, A2, A3, A4, A5, A6 or a custom size. Custom size can be set in px (pixels), mm or in (inches) with width and height separated by space like this: 200 300, 200px 300px, 200mm 300mm, 20cm 30cm or 6in 8in. (Optional)",
        default="",
    ),
    orientation: str = Field(
        description="Set to Portrait or Landscape. Portrait is set by default. (Optional)",
        default="",
    ),
    printBackground: bool = Field(
        description="true by default. Set to false to disable printing of background. (Optional)",
        default=True,
    ),
    mediaType: str = Field(
        description="Uses print by default. Set to screen to convert HTML as it appears in a browser or print to convert as it appears for printing or none to set none as mediaType for CSS styles. (Optional)",
        default="",
    ),
    DoNotWaitFullLoad: bool = Field(
        description="false by default. Set to true to skip waiting for full load (like full video load etc. that may affect the total conversion time). (Optional)",
        default=False,
    ),
    header: str = Field(
        description="User definable HTML for the header to be applied on every page header. (Optional)",
        default="",
    ),
    footer: str = Field(
        description="User definable HTML for the footer to be applied on every page footer. (Optional)",
        default="",
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert external webpage URL to PDF.
    Ref: https://developer.pdf.co/api-reference/pdf-from-url.md

    The header and footer parameters can contain valid HTML markup with the following classes used to inject printing values into them:
    - date: formatted print date
    - title: document title
    - url: document location
    - pageNumber: current page number
    - totalPages: total pages in the document
    - img: tag is supported in both the header and footer parameter, provided that the src attribute is specified as a base64-encoded string.
    For example, the following markup will generate Page N of NN page numbering:
    ```html
    <span style='font-size:10px'>Page <span class='pageNumber'></span> of <span class='totalPages'></span>.</span>
    """
    return await convert_from(
        "pdf",
        "url",
        ConversionParams(
            url=url,
            margins=margins,
            paperSize=paperSize,
            orientation=orientation,
            printBackground=printBackground,
            mediaType=mediaType,
            DoNotWaitFullLoad=DoNotWaitFullLoad,
            header=header,
            footer=footer,
            httpusername=httpusername,
            httppassword=httppassword,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def html_to_pdf(
    html: str = Field(
        description="Input HTML code to be converted. To convert the link to a PDF use the /pdf/convert/from/url endpoint instead. If it is a local file, just pass the file content as a string."
    ),
    margins: str = Field(
        description="Set to CSS style margins like 10px, 5mm, 5in for all sides or 5px 5px 5px 5px (the order of margins is top, right, bottom, left). (Optional)",
        default="",
    ),
    paperSize: str = Field(
        description="A4 is set by default. Can be Letter, Legal, Tabloid, Ledger, A0, A1, A2, A3, A4, A5, A6 or a custom size. Custom size can be set in px (pixels), mm or in (inches) with width and height separated by space like this: 200 300, 200px 300px, 200mm 300mm, 20cm 30cm or 6in 8in. (Optional)",
        default="",
    ),
    orientation: str = Field(
        description="Set to Portrait or Landscape. Portrait is set by default. (Optional)",
        default="",
    ),
    printBackground: bool = Field(
        description="true by default. Set to false to disable printing of background. (Optional)",
        default=True,
    ),
    mediaType: str = Field(
        description="Uses print by default. Set to screen to convert HTML as it appears in a browser or print to convert as it appears for printing or none to set none as mediaType for CSS styles. (Optional)",
        default="",
    ),
    DoNotWaitFullLoad: bool = Field(
        description="false by default. Set to true to skip waiting for full load (like full video load etc. that may affect the total conversion time). (Optional)",
        default=False,
    ),
    header: str = Field(
        description="User definable HTML for the header to be applied on every page header. (Optional)",
        default="",
    ),
    footer: str = Field(
        description="User definable HTML for the footer to be applied on every page footer. (Optional)",
        default="",
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert HTML to PDF.
    Ref: https://developer.pdf.co/api-reference/pdf-from-html/convert.md

    The header and footer parameters can contain valid HTML markup with the following classes used to inject printing values into them:
    - date: formatted print date
    - title: document title
    - url: document location
    - pageNumber: current page number
    - totalPages: total pages in the document
    - img: tag is supported in both the header and footer parameter, provided that the src attribute is specified as a base64-encoded string.
    For example, the following markup will generate Page N of NN page numbering:
    ```html
    <span style='font-size:10px'>Page <span class='pageNumber'></span> of <span class='totalPages'></span>.</span>
    """
    return await convert_from(
        "pdf",
        "html",
        ConversionParams(
            html=html,
            margins=margins,
            paperSize=paperSize,
            orientation=orientation,
            printBackground=printBackground,
            mediaType=mediaType,
            DoNotWaitFullLoad=DoNotWaitFullLoad,
            header=header,
            footer=footer,
            httpusername=httpusername,
            httppassword=httppassword,
            name=name,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def email_to_pdf(
    url: str = Field(
        description="URL to the source file (MSG, EML). Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    embedAttachments: bool = Field(
        description="Set to true to automatically embeds all attachments from original input email MSG or EML files into the final output PDF. Set it to false if you don’t want to embed attachments so it will convert only the body of the input email. True by default.",
        default=True,
    ),
    convertAttachments: bool = Field(
        description="Set to false if you don’t want to convert attachments from the original email and want to embed them as original files (as embedded PDF attachments). Converts attachments that are supported by the PDF.co API (DOC, DOCx, HTML, PNG, JPG etc.) into PDF format and then merges into output final PDF. Non-supported file types are added as PDF attachments (Adobe Reader or another viewer may be required to view PDF attachments).",
        default=True,
    ),
    margins: str = Field(
        description="Set to CSS style margins like 10px, 5mm, 5in for all sides or 5px 5px 5px 5px (the order of margins is top, right, bottom, left). (Optional)",
        default="",
    ),
    paperSize: str = Field(
        description="A4 is set by default. Can be Letter, Legal, Tabloid, Ledger, A0, A1, A2, A3, A4, A5, A6 or a custom size. Custom size can be set in px (pixels), mm or in (inches) with width and height separated by space like this: 200 300, 200px 300px, 200mm 300mm, 20cm 30cm or 6in 8in. (Optional)",
        default="",
    ),
    orientation: str = Field(
        description="Set to Portrait or Landscape. Portrait is set by default. (Optional)",
        default="",
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert email to PDF.
    Ref: https://developer.pdf.co/api-reference/pdf-from-email.md
    """
    return await convert_from(
        "pdf",
        "email",
        ConversionParams(
            url=url,
            embedAttachments=embedAttachments,
            convertAttachments=convertAttachments,
            margins=margins,
            paperSize=paperSize,
            orientation=orientation,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def excel_to_csv(
    url: str = Field(
        description="URL to the source file (XLS, XLSX). Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    worksheetIndex: str = Field(
        description="Index of the worksheet to convert. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert Excel(XLS, XLSX) to CSV.
    Ref: https://developer.pdf.co/api-reference/convert-from-excel/csv.md
    """
    return await convert_to(
        "xls",
        "csv",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            name=name,
            worksheetIndex=worksheetIndex,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def excel_to_json(
    url: str = Field(
        description="URL to the source file (XLS, XLSX). Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    worksheetIndex: str = Field(
        description="Index of the worksheet to convert. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert Excel(XLS, XLSX) to JSON.
    Ref: https://developer.pdf.co/api-reference/convert-from-excel/json.md
    """
    return await convert_to(
        "xls",
        "json",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            name=name,
            worksheetIndex=worksheetIndex,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def excel_to_html(
    url: str = Field(
        description="URL to the source file (XLS, XLSX). Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    worksheetIndex: str = Field(
        description="Index of the worksheet to convert. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert Excel(XLS, XLSX) to HTML.
    Ref: https://developer.pdf.co/api-reference/convert-from-excel/html.md
    """
    return await convert_to(
        "xls",
        "html",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            name=name,
            worksheetIndex=worksheetIndex,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def excel_to_txt(
    url: str = Field(
        description="URL to the source file (XLS, XLSX). Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    worksheetIndex: str = Field(
        description="Index of the worksheet to convert. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert Excel(XLS, XLSX) to TXT.
    Ref: https://developer.pdf.co/api-reference/convert-from-excel/text.md
    """
    return await convert_to(
        "xls",
        "txt",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            name=name,
            worksheetIndex=worksheetIndex,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def excel_to_xml(
    url: str = Field(
        description="URL to the source file (XLS, XLSX). Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    worksheetIndex: str = Field(
        description="Index of the worksheet to convert. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert Excel(XLS, XLSX) to XML.
    Ref: https://developer.pdf.co/api-reference/convert-from-excel/xml.md
    """
    return await convert_to(
        "xls",
        "xml",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            name=name,
            worksheetIndex=worksheetIndex,
            api_key=api_key,
        ),
    )


@mcp.tool()
async def excel_to_pdf(
    url: str = Field(
        description="URL to the source file (XLS, XLSX). Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    worksheetIndex: str = Field(
        description="Index of the worksheet to convert. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Convert Excel(XLS, XLSX) to PDF.
    Ref: https://developer.pdf.co/api-reference/convert-from-excel/pdf.md
    """
    return await convert_to(
        "xls",
        "pdf",
        ConversionParams(
            url=url,
            httpusername=httpusername,
            httppassword=httppassword,
            name=name,
            worksheetIndex=worksheetIndex,
            api_key=api_key,
        ),
    )



from pdfco.mcp.server import mcp
from pdfco.mcp.services.pdf import add_pdf_password, remove_pdf_password
from pdfco.mcp.models import BaseResponse, ConversionParams

from pydantic import Field


@mcp.tool()
async def pdf_add_password(
    url: str = Field(
        description="URL to the source PDF file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    owner_password: str = Field(
        description="The main owner password that is used for document encryption and for setting/removing restrictions."
    ),
    user_password: str = Field(
        description="The optional user password will be asked for viewing and printing document.",
        default="",
    ),
    encryption_algorithm: str = Field(
        description="Encryption algorithm. Valid values: RC4_40bit, RC4_128bit, AES_128bit, AES_256bit. AES_128bit or higher is recommended.",
        default="AES_256bit",
    ),
    allow_accessibility_support: bool = Field(
        description="Allow or prohibit content extraction for accessibility needs.",
        default=False,
    ),
    allow_assembly_document: bool = Field(
        description="Allow or prohibit assembling the document.", default=False
    ),
    allow_print_document: bool = Field(
        description="Allow or prohibit printing PDF document.", default=False
    ),
    allow_fill_forms: bool = Field(
        description="Allow or prohibit the filling of interactive form fields (including signature fields) in the PDF documents.",
        default=False,
    ),
    allow_modify_document: bool = Field(
        description="Allow or prohibit modification of PDF document.", default=False
    ),
    allow_content_extraction: bool = Field(
        description="Allow or prohibit copying content from PDF document.",
        default=False,
    ),
    allow_modify_annotations: bool = Field(
        description="Allow or prohibit interacting with text annotations and forms in PDF document.",
        default=False,
    ),
    print_quality: str = Field(
        description="Allowed printing quality. Valid values: HighResolution, LowResolution.",
        default="",
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    password: str = Field(
        description="Password of the PDF file if it's already password-protected. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Add password protection to a PDF file.
    Ref: https://developer.pdf.co/api-reference/pdf-password/add.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername,
        httppassword=httppassword,
        password=password,
        name=name,
    )

    additional_params = {
        "ownerPassword": owner_password,
    }

    if user_password is not None:
        additional_params["userPassword"] = user_password

    if encryption_algorithm is not None:
        additional_params["EncryptionAlgorithm"] = encryption_algorithm

    if allow_accessibility_support is not None:
        additional_params["AllowAccessibilitySupport"] = allow_accessibility_support

    if allow_assembly_document is not None:
        additional_params["AllowAssemblyDocument"] = allow_assembly_document

    if allow_print_document is not None:
        additional_params["AllowPrintDocument"] = allow_print_document

    if allow_fill_forms is not None:
        additional_params["AllowFillForms"] = allow_fill_forms

    if allow_modify_document is not None:
        additional_params["AllowModifyDocument"] = allow_modify_document

    if allow_content_extraction is not None:
        additional_params["AllowContentExtraction"] = allow_content_extraction

    if allow_modify_annotations is not None:
        additional_params["AllowModifyAnnotations"] = allow_modify_annotations

    if print_quality is not None:
        additional_params["PrintQuality"] = print_quality

    return await add_pdf_password(params, **additional_params, api_key=api_key)


@mcp.tool()
async def pdf_remove_password(
    url: str = Field(
        description="URL to the source PDF file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    password: str = Field(
        description="Password of the PDF file to be removed. (Optional)", default=""
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Remove password protection from a PDF file.
    Ref: https://developer.pdf.co/api-reference/pdf-password/remove.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername,
        httppassword=httppassword,
        password=password,
        name=name,
    )

    return await remove_pdf_password(params, api_key=api_key)



from pdfco.mcp.server import mcp
from pdfco.mcp.services.pdf import find_text_in_pdf, find_table_in_pdf
from pdfco.mcp.models import BaseResponse, ConversionParams

from pydantic import Field


@mcp.tool(name="find_text")
async def find_text(
    url: str = Field(
        description="URL to the source PDF file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    searchString: str = Field(
        description="Text to search. Can support regular expressions if regexSearch is set to True."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated list of page indices (or ranges) to process. Leave empty for all pages. Example: '0,2-5,7-'. The first-page index is 0. (Optional)",
        default="",
    ),
    wordMatchingMode: str = Field(
        description="Values can be either SmartMatch, ExactMatch, or None. (Optional)",
        default=None,
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    regexSearch: bool = Field(
        description="Set to True to enable regular expressions in the search string. (Optional)",
        default=False,
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Find text in PDF and get coordinates. Supports regular expressions.
    Ref: https://developer.pdf.co/api-reference/pdf-find/basic.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername,
        httppassword=httppassword,
        pages=pages,
        password=password,
    )

    return await find_text_in_pdf(
        params, searchString, regexSearch, wordMatchingMode, api_key=api_key
    )


@mcp.tool(name="find_table")
async def find_table(
    url: str = Field(
        description="URL to the source PDF file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    pages: str = Field(
        description="Comma-separated list of page indices (or ranges) to process. Leave empty for all pages. Example: '0,2-5,7-'. The first-page index is 0. (Optional)",
        default="",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Find tables in PDF and get their coordinates.
    Ref: https://developer.pdf.co/api-reference/pdf-find/table.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername,
        httppassword=httppassword,
        pages=pages,
        password=password,
    )

    return await find_table_in_pdf(params, api_key=api_key)



from pdfco.mcp.server import mcp
from pdfco.mcp.services.pdf import get_pdf_form_fields_info, fill_pdf_form_fields
from pdfco.mcp.models import BaseResponse, ConversionParams

from pydantic import Field


@mcp.tool()
async def read_pdf_forms_info(
    url: str = Field(
        description="URL to the source PDF file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    password: str = Field(description="Password of PDF file. (Optional)", default=""),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Extracts information about fillable PDF fields from an input PDF file.
    Ref: https://developer.pdf.co/api-reference/forms/info-reader.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername,
        httppassword=httppassword,
        password=password,
    )

    return await get_pdf_form_fields_info(params, api_key=api_key)


@mcp.tool(name="fill_forms")
async def fill_pdf_forms(
    url: str = Field(
        description="URL to the source PDF file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    fields: list = Field(
        description="List of fields to fill. Each field is a dict with 'fieldName', 'pages', and 'text' properties."
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Fill existing form fields in a PDF document.

    Example fields format:
    [
        {
            "fieldName": "field_name_from_form_info",
            "pages": "1",
            "text": "Value to fill"
        }
    ]

    Use 'read_pdf_forms_info' first to get the fieldName values of the form.

    Ref: https://developer.pdf.co/api-reference/pdf-add#create-fillable-pdf-forms.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername,
        httppassword=httppassword,
        name=name,
    )

    return await fill_pdf_form_fields(params, fields=fields, api_key=api_key)


@mcp.tool(name="create_fillable_forms")
async def create_fillable_forms(
    url: str = Field(
        description="URL to the source PDF file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    annotations: list = Field(
        description="List of form annotations to create. Each annotation can be a textfield or checkbox with properties like 'x', 'y', 'size', 'pages', 'type', and 'id'."
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Create new fillable form elements in a PDF document.

    Example annotations format:
    [
        {
            "text": "prefilled text",
            "x": 10,
            "y": 30,
            "size": 12,
            "pages": "0-",
            "type": "TextField",
            "id": "textfield1"
        },
        {
            "x": 100,
            "y": 150,
            "size": 12,
            "pages": "0-",
            "type": "Checkbox",
            "id": "checkbox1"
        }
    ]

    Ref: https://developer.pdf.co/api-reference/pdf-add#create-fillable-pdf-forms.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername,
        httppassword=httppassword,
        name=name,
    )

    return await fill_pdf_form_fields(params, annotations=annotations, api_key=api_key)



import sys
from pdfco.mcp.server import mcp
from pdfco.mcp.tools.apis import (
    conversion,
    job,
    file,
    modification,
    form,
    search,
    searchable,
    security,
    document,
    extraction,
    editing,
)


def main():
    if len(sys.argv) > 1:
        transport = sys.argv[1]
        if transport == "stdio":
            mcp.run(transport=transport)
        elif transport == "sse":
            if len(sys.argv) < 2:
                raise ValueError("SSE transport requires a port number")
            port = int(sys.argv[2])
            mcp.run(transport=transport, host="0.0.0.0", port=port)
        elif transport == "streamable-http":
            if len(sys.argv) < 3:
                raise ValueError(
                    "Streamable HTTP transport requires a port number and path"
                )
            port = int(sys.argv[2])
            path = sys.argv[3]
            mcp.run(transport=transport, host="0.0.0.0", port=port, path=path)
        else:
            raise ValueError(f"Invalid transport: {transport}")
    else:
        mcp.run(transport="stdio")


if __name__ == "__main__":
    main()



import sys
from pdfco.mcp.models import BaseResponse, ConversionParams
from pdfco.mcp.services.client import PDFCoClient


async def convert_to(
    _from: str, _to: str, params: ConversionParams, api_key: str | None = None
) -> BaseResponse:
    return await request(f"{_from}/convert/to/{_to}", params, api_key=api_key)


async def convert_from(
    _to: str, _from: str, params: ConversionParams, api_key: str | None = None
) -> BaseResponse:
    return await request(f"{_to}/convert/from/{_from}", params, api_key=api_key)


async def merge_pdf(params: ConversionParams, api_key: str | None = None) -> BaseResponse:
    return await request("pdf/merge2", params, api_key=api_key)


async def split_pdf(params: ConversionParams, api_key: str | None = None) -> BaseResponse:
    return await request("pdf/split", params, api_key=api_key)


async def get_pdf_form_fields_info(
    params: ConversionParams, api_key: str | None = None
) -> BaseResponse:
    return await request("pdf/info/fields", params, api_key=api_key)


async def fill_pdf_form_fields(
    params: ConversionParams,
    fields: list | None = None,
    annotations: list | None = None,
    api_key: str | None = None,
) -> BaseResponse:
    custom_payload = {}
    if fields:
        custom_payload["fields"] = fields
    if annotations:
        custom_payload["annotations"] = annotations
    return await request(
        "pdf/edit/add", params, custom_payload=custom_payload, api_key=api_key
    )


async def pdf_add(
    params: ConversionParams, api_key: str | None = None, **kwargs
) -> BaseResponse:
    """General PDF Add function that supports all PDF Add API parameters"""
    custom_payload = {}

    # Add all supported parameters
    for key, value in kwargs.items():
        if value is not None and value != "":
            custom_payload[key] = value

    return await request(
        "pdf/edit/add", params, custom_payload=custom_payload, api_key=api_key
    )


async def find_text_in_pdf(
    params: ConversionParams,
    search_string: str,
    regex_search: bool = False,
    word_matching_mode: str | None = None,
    api_key: str | None = None,
) -> BaseResponse:
    custom_payload = {"searchString": search_string, "regexSearch": regex_search}
    if word_matching_mode:
        custom_payload["wordMatchingMode"] = word_matching_mode
    return await request(
        "pdf/find", params, custom_payload=custom_payload, api_key=api_key
    )


async def find_table_in_pdf(
    params: ConversionParams, api_key: str | None = None
) -> BaseResponse:
    return await request("pdf/find/table", params, api_key=api_key)


async def make_pdf_searchable(
    params: ConversionParams, api_key: str | None = None
) -> BaseResponse:
    return await request("pdf/makesearchable", params, api_key=api_key)


async def make_pdf_unsearchable(
    params: ConversionParams, api_key: str | None = None
) -> BaseResponse:
    return await request("pdf/makeunsearchable", params, api_key=api_key)


async def get_pdf_info(params: ConversionParams, api_key: str | None = None) -> BaseResponse:
    return await request("pdf/info", params, api_key=api_key)


async def add_pdf_password(
    params: ConversionParams, api_key: str | None = None, **kwargs
) -> BaseResponse:
    return await request(
        "pdf/security/add", params, custom_payload=kwargs, api_key=api_key
    )


async def remove_pdf_password(
    params: ConversionParams, api_key: str | None = None
) -> BaseResponse:
    return await request("pdf/security/remove", params, api_key=api_key)


async def parse_invoice(params: ConversionParams, api_key: str | None = None) -> BaseResponse:
    return await request("ai-invoice-parser", params, api_key=api_key)


async def extract_pdf_attachments(
    params: ConversionParams, api_key: str | None = None
) -> BaseResponse:
    return await request("pdf/attachments/extract", params, api_key=api_key)


async def request(
    endpoint: str,
    params: ConversionParams,
    custom_payload: dict | None = None,
    api_key: str | None = None,
) -> BaseResponse:
    payload = params.parse_payload(async_mode=True)
    if custom_payload:
        payload.update(custom_payload)

    try:
        async with PDFCoClient(api_key=api_key) as client:
            url = f"/v1/{endpoint}"
            print(f"Requesting {url} with payload {payload}", file=sys.stderr)
            response = await client.post(url, json=payload)
            print(f"response: {response}", file=sys.stderr)
            json_data = response.json()
            return BaseResponse(
                status="working",
                content=json_data,
                credits_used=json_data.get("credits"),
                credits_remaining=json_data.get("remainingCredits"),
                tips=f"You **should** use the 'wait_job_completion' tool to wait for the job [{json_data.get('jobId')}] to complete if a jobId is present.",
            )
    except Exception as e:
        return BaseResponse(
            status="error",
            content=f"{type(e)}: {[arg for arg in e.args if arg]}",
        )



from pdfco.mcp.server import mcp
from pdfco.mcp.services.pdf import merge_pdf, split_pdf
from pdfco.mcp.models import BaseResponse, ConversionParams

from pydantic import Field


@mcp.tool()
async def pdf_merge(
    url: str = Field(
        description="URLs to the source files as a comma-separated list. Supports PDF, DOC, DOCX, RTF, TXT, XLS, XLSX, CSV, images, and more. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    name: str = Field(
        description="File name for the generated output. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Merge PDF from two or more PDF, DOC, XLS, images, even ZIP with documents and images into a new PDF.
    Ref: https://developer.pdf.co/api-reference/merge/various-files.md
    """
    return await merge_pdf(
        ConversionParams(
            url=url, httpusername=httpusername, httppassword=httppassword, name=name
        ),
        api_key=api_key,
    )


@mcp.tool()
async def pdf_split(
    url: str = Field(
        description="URL to the source PDF file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    pages: str = Field(
        description="Comma-separated indices of pages (or page ranges) that you want to use. The first-page index is 1. For example: '1,3,5-7' or '1-2,4-'. Use '*' to split every page into separate files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    name: str = Field(
        description="Base file name for the generated output files. (Optional)",
        default="",
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Split a PDF into multiple PDF files using page indexes or page ranges.
    Ref: https://developer.pdf.co/api-reference/pdf-split/by-pages.md
    """
    params = ConversionParams(
        url=url,
        pages=pages,
        httpusername=httpusername,
        httppassword=httppassword,
        password=password,
        name=name,
    )

    return await split_pdf(params, api_key=api_key)



from pdfco.mcp.server import mcp
from pdfco.mcp.services.pdf import get_pdf_info
from pdfco.mcp.models import BaseResponse, ConversionParams

from pydantic import Field


@mcp.tool()
async def pdf_info_reader(
    url: str = Field(
        description="URL to the source PDF file. Supports publicly accessible links including Google Drive, Dropbox, PDF.co Built-In Files Storage. Use 'upload_file' tool to upload local files."
    ),
    httpusername: str = Field(
        description="HTTP auth user name if required to access source url. (Optional)",
        default="",
    ),
    httppassword: str = Field(
        description="HTTP auth password if required to access source url. (Optional)",
        default="",
    ),
    password: str = Field(
        description="Password of the PDF file. (Optional)", default=""
    ),
    api_key: str = Field(
        description="PDF.co API key. If not provided, will use X_API_KEY environment variable. (Optional)",
        default="",
    ),
) -> BaseResponse:
    """
    Get detailed information about a PDF document - number of pages, metadata, security, form fields, and more.
    Ref: https://developer.pdf.co/api-reference/pdf-info-reader.md
    """
    params = ConversionParams(
        url=url,
        httpusername=httpusername,
        httppassword=httppassword,
        password=password,
    )

    return await get_pdf_info(params, api_key=api_key)
